/* Soubor je ulozen v kodovani UTF-8.
 * Kontrola kódování: Příliš žluťoučký kůň úpěl ďábelské ódy.
 */
package cz.pecinovsky.english.pojj.adv_framework.scenario;

import cz.pecinovsky.english.pojj.adv_framework.game_txt.INamed;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;



/*******************************************************************************
 * Instance třídy {@code Scenario} představují scénáře,
 * podle kterých je možno hrát hru, pro kterou jsou určeny.
 * Aby bylo možno jednotlivé scénáře od sebe odlišit,
 * je každý pojmenován a má přiřazen typ, podle které lze blíže určit,
 * k čemu je možno daný scénář použít.
 *
 * @author    Rudolf PECINOVSKÝ
 * @version   0.00.000
 */
public class Scenario implements INamed, Iterable<ScenarioStep>
{
//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================
//== PROMĚNNÉ ATRIBUTY TŘÍDY ===================================================
//== STATICKÝ INICIALIZAČNÍ BLOK - STATICKÝ KONSTRUKTOR ========================
//== KONSTANTNÍ ATRIBUTY INSTANCÍ ==============================================

    /** Název scénáře. */
    private final String name;

    /** Typ scénáře. */
    private final TypeOfScenario type;

    /** Třída hry. */
    private final AScenarioManager scenarioManager;

    /** Jednotlivé kroky daného scénáře. */
    private final List<ScenarioStep> steps = new ArrayList<ScenarioStep>();



//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================

    /** Podpis scénáře - vyrábí se až na požádání. */
    private String podpis;



//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ TŘÍDY ========================================
//== OSTATNÍ NESOUKROMÉ METODY TŘÍDY ===========================================

//##############################################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY =============================================

    /***************************************************************************
     * Vytvoří nový scénář zadaného názvu a typu určený pro zadnou hru.
     * Konstruktor není veřejný, protože bude volán pouze z metod třídy
     * {@link AScenarioManager}, která je ve stejném balíčku.
     *
     * @param name  Název scénáře. Název musí být neprázdný řetězec
     *              a nepočítají se do něj případné úvodní a závěrečné mezery.
     * @param type  Typ scénáře
     * @param scenarioManager Správce, jenž bude vytvořený scénář spravovat
     *              (a který si jej pro sebe vytváří)
     * @param steps Jednotlivé kroky realizující daný scénář;
     *              příkaz v prvním (přesněji nutlém) kroku
     *              musí být definována jako prázdný řetězec
     */
    @SuppressWarnings("AssignmentToMethodParameter")
    public Scenario(String name, TypeOfScenario type,
                    AScenarioManager scenarioManager, ScenarioStep... steps)
    {
        name = name.trim();
        if ((name  == null)  ||  ("".equals(name))   ||  (type == null)  ||
            (steps == null)  ||  (steps.length == 0) ||
            (! "".equals(steps[0].command)))
        {
            throw new IllegalArgumentException(
                "\nHodnoty parametrů konstruktoru neodpovídají kontraktu:" +
                "\n   Název: «" + name +
               "»\n   Kroků: "  + (steps==null ? "NULL" : steps.length) +
                "\n   Startovací příkaz: «" +
                      ((steps==null || steps.length==0) ? "NULL"
                                                        : steps[0].command) + "»"
                );
        }
        this.name = name;
        this.type = type;
        this.scenarioManager = scenarioManager;
        this.steps.addAll(Arrays.asList(steps));
    }


    /***************************************************************************
     * Vytvoří nový demonstrační scénář zadaného názvu, který bude obsahovat
     * pouze zadání příkazů a nebude umožňovat kontrolu správných reakcí hry.
     * Konstruktor není veřejný, protože jej bude volat pouze správce scénářů,
     * který si scénář vytváří pro sebe.
     * <p>
     * Parametry musí odpovídat kontraktu uvedenému u čtyřparametrické verze.
     *
     * @param name             Název scénáře v rámci scénářů dané hry
     * @param scenarioManager  Třída hry, pro níž je scénář vytvářen
     * @param command          Posloupnost akcí realizujících daný scénář
     */
    Scenario(String name, AScenarioManager scenarioManager,
             String... command)
    {
        this(name, TypeOfScenario.scDEMO, scenarioManager,
             prepareSteps(command));
    }



//== ABSTRAKTNÍ METODY =========================================================
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ INSTANCÍ =====================================

    /***************************************************************************
     * Vrátí název scénáře.
     *
     * @return Název scénáře
     */
    @Override
    public String getName()
    {
        return name;
    }


    /***************************************************************************
     * Vrátí typ scénáře.
     *
     * @return  Typ scénáře
     */
    public TypeOfScenario getType()
    {
        return type;
    }


    /***************************************************************************
     * Vrátí svého správce scénářů.
     *
     * @return  Správce scénářů spravující a zpříástupňující daný scénář
     */
    public AScenarioManager getManager()
    {
        return scenarioManager;
    }


    /***************************************************************************
     * Vrátí řetězec s jednotlivými příkazy scénáře.
     * Každý příkaz bude na samostatném řádku uvozený zadaným prefixem.
     *
     * @param  prefix  Začátek řádku předcházející každému příkazu
     * @return Řetězec s jednotlivými příkazy scénáře
     */
    public String getCommands(String prefix)
    {
        StringBuilder sb = new StringBuilder();
        for (ScenarioStep step : this) {
            sb.append(prefix).append(step.command).append('\n');
        }
        return sb.toString();
    }


    /***************************************************************************
     * Vrátí název daného scénáře.
     *
     * @return Nazev daného scénáře
     */
    public List<String> getListOfCommands()
    {
        List<String> list = new ArrayList<String>();
        for (ScenarioStep step : this) {
            list.add(step.command);
        }
        return list;
    }



//== OSTATNÍ NESOUKROMÉ METODY INSTANCÍ ========================================

    /***************************************************************************
     * Vrátí iterátor umožňující procházet kroky daného scénáře.
     *
     * @return  Iterátor kroků scénáře
     */
    @Override
    public Iterator<ScenarioStep> iterator()
    {
        return steps.iterator();
    }


    /***************************************************************************
     * Vrátí textový podpis daného scénáře,
     * v němž je uveden jeho index, název a správce.
     *
     * @return Textový podpis daného scénáře
     */
    @Override
    public String toString()
    {
        if (podpis == null) {
            AScenarioManager asm = getManager();
            podpis = "Scénář(Index=" + asm.getIndexOf(this) +
                     ", Název="      + getName() +
                     ", Správce="    + getManager() + ")";
        }
        return podpis;
    }



//== SOUKROMÉ A POMOCNÉ METODY TŘÍDY ===========================================

    /***************************************************************************
     * Ze zadaných popisů akcí vytvoří pole odpovídajících demonstračních
     * kroků scénáře, tj. kroků, které nebudou umožňovat testování,
     * ale budou sloužit pouze k simulaci průběhu hry.
     *
     * @param  command Pole popisů zadávaných akcí
     * @return Pole demonstračních kroků scénáře
     */
    private static ScenarioStep[] prepareSteps(String[] command)
    {
        ScenarioStep.clearIndex();
        ScenarioStep[] steps = new ScenarioStep[command.length+1];
        steps[0] = new ScenarioStep("");
        for (int i=0;   i < command.length;   i++) {
            ScenarioStep step = new ScenarioStep(command[i]);
            steps[i+1] = step;
        }
        return steps;
    }



//== SOUKROMÉ A POMOCNÉ METODY INSTANCÍ ========================================
//== INTERNÍ DATOVÉ TYPY =======================================================
//== TESTY A METODA MAIN =======================================================
//
//    /***************************************************************************
//     * Testovací metoda.
//     */
//    public static void test()
//    {
//        Scenario inst = new Scenario();
//    }
//    /** @param args Parametry příkazového řádku - nepoužívané. */
//    public static void main( String[] args )  {  test();  }
}
