/* Soubor je ulozen v kodovani UTF-8.
 * Kontrola kódování: Příliš žluťoučký kůň úpěl ďábelské ódy.
 */
package cz.pecinovsky.english.pojj.adv_framework.test_util;

import cz.pecinovsky.english.pojj.adv_framework.game_gui.IGameG;
import cz.pecinovsky.english.pojj.adv_framework.game_gui.IUIG;

import cz.pecinovsky.english.pojj.adv_framework.game_txt.IGame;

import cz.pecinovsky.english.pojj.adv_framework.scenario.AScenarioManager;
import cz.pecinovsky.english.pojj.adv_framework.scenario.Scenario;
import cz.pecinovsky.english.pojj.adv_framework.test_util.default_game.hra.DefaultGame;
import cz.pecinovsky.english.pojj.adv_framework.test_util.default_gui.DefaultGUI;



/*******************************************************************************
 * Třída {@code Triumvirate} je společným rodičem všech tříd, které pracují
 * s triumvirátem [správce scénářů - třída hry - hra].
 * Konstruktor jejich instancí prověří, zda je trojice konzistentní.
 * Vytvořená instance poskytuje metody na získání jednotlivých členů trojice.
 *
 * @author    Rudolf PECINOVSKÝ
 * @version   0.00.000
 */
public abstract class Triumvirate
{
//== KONSTANTNÍ ATRIBUTY TŘÍDY =================================================
//== PROMĚNNÉ ATRIBUTY TŘÍDY ===================================================
//== STATICKÝ INICIALIZAČNÍ BLOK - STATICKÝ KONSTRUKTOR ========================
//== KONSTANTNÍ ATRIBUTY INSTANCÍ ==============================================

    /** Správce scénářů, který spravuje scénáře umožňující testování hry. */
    final AScenarioManager manager;

    /** Třída, jejíž instance představují hry,
     *  které je možno hrát podle spravovaných scénářů. */
    final Class<? extends IGame> gameClass;

    /** Konkrétní hra (instance třídy {@link gameClass}, která je/byla/bude
     *  právě testována prostřednictvím scénářů správce. */
    final IGame gameT;

    /** Implementuje-li hra rozhraní {@link IGameG}, naplní se  další atribut
     *  přetypovanou instancí, aby potomci nemuseli přetypovávat. */
    final IGameG gameG;



//== PROMĚNNÉ ATRIBUTY INSTANCÍ ================================================
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ TŘÍDY ========================================

    /***************************************************************************
     * Vrátí instanci implicitní hry implementující rozhraní {@link IGame}.
     *
     * @return Instance implicitní hry
     */
    public static IGameG getDefaultGame()
    {
        return DefaultGame.getInstance();
    }


    /***************************************************************************
     * Vrátí novou instanci implicitního grafického rozhraní
     * implementujícího rozhraní {@link IUIG}.
     *
     * @return Nová instance implicitního grafického rozhraní
     */
    public static IUIG getDefaultGUI()
    {
        return new DefaultGUI();
    }



//== OSTATNÍ NESOUKROMÉ METODY TŘÍDY ===========================================

    /***************************************************************************
     * Vrátí instanci testovací třídy napojenou na zadaného správce scénářů.
     * Správce scénářů je klíčovým bodem celé hry, protože jeho prostřednictvím
     * se je možno dostat ke všem ostatním dostupným informacím o hře.
     *
     * @param manager Správce scénářů prověřované aplikace
     * @return Instance schopná testovat zadého správce scénářů
     *         včetně případné hry, jejíž scénáře spravuje
     */
    static Crate fromManager(AScenarioManager manager)
    {
        AScenarioManager       sm = manager;
        Class<? extends IGame> gc = sm.getGameClass();
        IGame                  gm = sm.getGame();
        return new Crate(sm, gc, gm);
    }


    /***************************************************************************
     * Vrátí instanci testovací třídy schopnou testovat
     * instance zadané třídy hry.
     *
     * @param gameClass Třída testované hry
     * @return Instance schopná testovat zadanou třídu hry a její hru
     */
    static Crate fromGameClass(Class<? extends IGame> gameClass)
    {
        Class<? extends IGame> gc = gameClass;
        IGame                  gm = GameTClassTest.getInstanceOfGame(gc);
        AScenarioManager       sm = gm.getScenarioManager();
        return new Crate(sm, gc, gm);
    }


    /***************************************************************************
     * Vrátí instanci testovací třídy schopnou testovat zadanou hru.
     *
     * @param game Testovaná hra
     * @return Instance schopná testovat zadanou hru
     */
    static Crate fromGame(IGame game)
    {
        IGame                  gm = game;
        Class<? extends IGame> gc = game.getClass();
        AScenarioManager       sm = game.getScenarioManager();
        return new Crate(sm, gc, gm);
    }


    /***************************************************************************
     * Ověří, že zadané tři parametry si navzájem odpovídají, tj. že:
     * <ul>
     *   <li>je zadán správce scénářů,</li>
     *   <li>hra a třída hry musejí být zadány obě nebo žádná,</li>
     *   <li>zadaná hra je instancí zadané třídy hry,</li>
     *   <li>zadaná třída hry vrací zadanou hru, která je jedináčkem,</li>
     *   <li>zadaná hra vrací zadaného správce scénářů,</li>
     *   <li>správce scénářů hlásí napojení na zadanou třídu hry,</li>
     *   <li>správce scénářů hlásí napojení na zadanou instanci hry,</li>
     *   <li>jméno a x-name autora hry musí být totožné
     *       se jménem a x-name autor spráce scénářů,</li>
     * </ul>
     *
     * @param manager   Správce scénářů prověřované aplikace
     * @param gameClass Třída testované hry
     * @param game      Testovaná instance
     * @throws IllegalArgumentException Nevyhovují-li parametry podmínkám
     */
    static void verifyTriumvirate(AScenarioManager manager,
                                  Class<? extends IGame> gameClass, IGame game)
           throws IllegalArgumentException
    {
        if (manager == null) {
            throw new IllegalArgumentException(
                  "\nNebyl zadán správce scénářů");
        }
        if ((gameClass == null)) {
            if(game != null) {
                throw new IllegalArgumentException(
                      "\nByla zadána instance hry, ale ne třída hry");
            }
            return;     //==========>
        }

        //gameClass != null
        if (game == null) {
            throw new IllegalArgumentException(
                  "\nByla zadána třída hry, ale ne instance hry");
        }
        if (game.getClass() != gameClass) {
            throw new IllegalArgumentException(
                  "\nZadaná třída hry se neshoduje s třídou zadané hry" +
                  "\n   Zadaná třída hry: " + gameClass +
                  "\n   Třída zadané hry: " + game.getClass());
        }
        if (game != GameTClassTest.getInstanceOfGame(gameClass))  {
            throw new IllegalArgumentException(
                  "\nZadaná hra není hrou, kterou vrací tovární metoda zadané" +
                  "třídy jako svého jedináčka\n   Třída: " + gameClass);
        }
        Class<? extends IGame> managerGameClass = manager.getGameClass();
        if  (gameClass != managerGameClass)  {
            throw new IllegalArgumentException(
                  "\nZadaná třída hry se neshoduje s třídou příslušnou " +
                     "k zadanému správce scénářů:" +
                  "\n   Třída, k níž se hlásí scénář: " + managerGameClass +
                  "\n   Zadaná třída hry:             " + gameClass);
        }
        IGame mGame = manager.getGame();
        if (game != mGame) {
            throw new IllegalArgumentException(
                  "\nInstance hry vrácená správcem scénářů není totožná " +
                  "se zadanou instancí hry: " + gameClass);
        }
        if (! (manager.getAuthor().equals(game.getAuthor())  &&
              (manager.getXNAME() .equals(game.getXNAME()))) )
        {
            throw new IllegalArgumentException(
               "\nAutor hry není totožný s autorem správce scénářů:" +
               "\n   Autoh správce scénářů: " + manager.getXNAME() + " - " +
                                                manager.getAuthor() +
               "\n   Autoh hry:             " + game.getXNAME() + " - " +
                                                game.getAuthor());
        }
    }


    /***************************************************************************
     * "Zahraje" podle zadaného scénáře, aniž by se snažila prověřit,
     * zda se hra chová opravdu tak, jak případný testovací scénář naplánoval.
     * Metodu lze proto použít i pro demonstrační scénáře.
     *
     * @param game     Hra, která se bude hrát podle zadaného scénáře
     * @param scenario Scénář, podle nějž se hra bude hrát
     * @param gui      Instance GUI, která bude zobrazovat průběh hry
     *                 a vůči níž bude testovací program vystupovat v roli hráče
     */
    public static void playGameByScenario(
                       IGame game, Scenario scenario, IUIG gui)
    {
        GameGRunTest.playGameByScenario(game, scenario, gui);
//        TestGUI test = new TestGUI(game.getClass());
//        test.test(gui, game, AScenarioManager.INDEX_GUI_TEST);
    }


    /***************************************************************************
     * "Zahraje" podle zadaného scénáře, aniž by se snažila prověřit,
     * zda se hra chová opravdu tak, jak případný testovací scénář naplánoval.
     * Metodu lze proto použít i pro demonstrační scénáře.
     *
     * @param game   Hra, která se bude hrát podle zadaného scénáře
     * @param index  Index scénáře, podle nějž se hra bude hrát
     * @param gui    Instance GUI, která bude zobrazovat průběh hry
     *               a vůči níž bude testovací program vystupovat v roli hráče
     */
    public static void playGameByScenario(IGame game, int index, IUIG gui)
    {
        Scenario scenario = game.getScenarioManager().getScenario(index);
        playGameByScenario(game, scenario, gui);
    }



//##############################################################################
//== KONSTRUKTORY A TOVÁRNÍ METODY =============================================

    /***************************************************************************
     * Na základě obsahu přepravky s trojicí klíčových objektů
     * vytvoří novou instanci základu testovacích objektů
     *
     * @param crate Přepravka s potřebnými objekty
     * @throws IllegalArgumentException Nevyhovují-li parametry podmínkám
     */
    Triumvirate(Crate crate)
    {
        this(crate.manager, crate.gameClass, crate.game);
    }


    /***************************************************************************
     * Na základě obsahu přepravky s trojicí klíčových objektů
     * vytvoří novou instanci základu testovacích objektů
     *
     * @param manager   Správce scénářů prověřované aplikace
     * @param gameClass Třída testované hry
     * @param game      Testovaná instance
     * @throws IllegalArgumentException Nevyhovují-li parametry podmínkám
     */
    Triumvirate(AScenarioManager manager,
                Class<? extends IGame> gameClass, IGame game)
    {
        this.manager   = manager;
        this.gameClass = gameClass;
        this.gameT     = game;
        this.gameG     = (game instanceof IGameG)  ?  (IGameG)game  :  null;
    }




//== ABSTRAKTNÍ METODY =========================================================
//== PŘÍSTUPOVÉ METODY VLASTNOSTÍ INSTANCÍ =====================================

    /***************************************************************************
     * Vrátí hru prověřovanou daným testem.
     *
     * @return Hra prověřovaná daným testem
     */
    public IGame getGame()
    {
        return gameT;
    }


    /***************************************************************************
     * Vrátí hru prověřovanou daným testem.
     *
     * @return Hra prověřovaná daným testem
     */
    public IGameG getGameG()
    {
        return gameG;
    }


    /***************************************************************************
     * Vrátí hru prověřovanou daným testem.
     *
     * @return Hra prověřovaná daným testem
     */
    Crate getCrate()
    {
        return new Crate(manager, gameClass, gameT);
    }


    /***************************************************************************
     * Vrátí třídu hry prověřované tímto testem.
     *
     * @return Třída hry prověřované tímto testem
     */
    public Class<? extends IGame> getGameClass()
    {
        return gameClass;
    }


    /***************************************************************************
     * Vrátí správce scénářů testované hry.
     *
     * @return Správce scénářů testované hry
     */
    public AScenarioManager getManager()
    {
        return manager;
    }



//== OSTATNÍ NESOUKROMÉ METODY INSTANCÍ ========================================

    /***************************************************************************
     * Ověří, že hra implementuje rozhraní {@link IGameG}.
     * Není-li tomu tak, vyhodí výjimku.
     *
     * @throws IllegalArgumentException
     *         Nemplementuje-li hra rozhraní {@link IGameG}
     */
    final void verifyImplementsIGameG()
    {
        if (gameG == null) {
            throw new IllegalArgumentException(
                "\nHra neimplementuje rozhraní IGameG: " + gameClass);
        }
    }



//== SOUKROMÉ A POMOCNÉ METODY TŘÍDY ===========================================
//== SOUKROMÉ A POMOCNÉ METODY INSTANCÍ ========================================
//== INTERNÍ DATOVÉ TYPY =======================================================

    static class Crate
    {
        final AScenarioManager       manager;
        final Class<? extends IGame> gameClass;
        final IGame                  game;


        Crate(AScenarioManager manager,
              Class<? extends IGame> gameClass, IGame game)
        {
            this.manager = manager;
            this.gameClass = gameClass;
            this.game = game;
        }
    }



//== TESTY A METODA MAIN =======================================================
//
//    /***************************************************************************
//     * Testovací metoda.
//     */
//    static void test()
//    {
//        Triumvirate inst = new Triumvirate();
//    }
//    /** @param args Parametry příkazového řádku - nepoužívané. */
//    public static void main( String[] args )  {  test();  }
}
